#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cublas.h"

// Taille des matrices
#define N (275)
#define N2 (N * N);

int main(int argc, char** argv)
{
	// Dclaration des variables
	cublasStatus status;
	float *	h_A;
	float *	h_B;
	float *	h_C;
	float *	d_A		= 0;
	float *	d_B		= 0;
	float *	d_C		= 0;
	float	alpha	= 1.0f;
	float	beta	= 0.0f;
	int		i;
	
	// Initialisation de CuBLAS
	// avec gestion des erreurs
	status = cublasInit();
	if (status != CUBLAS_STATUS_SUCCESS)
	{
		fprintf (stderr, "!!!!! Erreur lors de l'initialisation de CuBLAS !!!!!\n");
		return EXIT_FAILURE;
	}

	// Allocation de la mmoire pour les matrices sur l'hte
	// avec gestion des erreurs
	//h_A
	h_A = (float *) malloc ( N2 * sizeof(h_A[0]) );
	if (h_A == 0)
	{
		fprintf (stderr, "!!!!! Erreur lors de l'allocation mmoire de la matrice A sur l'hte !!!!!\n");
		return EXIT_FAILURE;
	}
	//h_B
	h_B = (float * ) malloc ( N2 * sizeof(h_B[0]) );
	if (h_B == 0)
	{
		fprintf (stderr, "!!!!! Erreur lors de l'allocation mmoire de la matrice B sur l'hte !!!!!\n");
		return EXIT_FAILURE;
	}
	//h_C
	h_C = (float * ) malloc ( N2 * sizeof(h_C[0]) );
	if (h_C == 0)
	{
		fprintf (stderr, "!!!!! Erreur lors de l'allocation mmoire de la matrice C sur l'hte !!!!!\n");
		return EXIT_FAILURE;
	}
	
	//Remplissage des matrices avec des donnes alatoires
	for (i = 0 ; i < N2 ; i++)
	{
		h_A[i] = rand() / (float) RAND_MAX;
		h_B[i] = rand() / (float) RAND_MAX;
		h_C[i] = rand() / (float) RAND_MAX;
	}
	
	// Allocation de la mmoire pour les matrices sur le priphrique
	// avec gestion des erreurs
	//d_A
	status = cublasAlloc (N2, sizeof (d_A[0]), (void **) & d_A);
	if (status != CUBLAS_STATUS_SUCCESS)
	{
		fprintf (stderr, "!!!!! Erreur lors de l'allocation mmoire de la matrice A sur le priphrique !!!!!\n");
		return EXIT_FAILURE;
	}
	//d_B
	status = cublasAlloc (N2, sizeof (d_B[0]), (void **) & d_B);
	if (status != CUBLAS_STATUS_SUCCESS)
	{
		fprintf (stderr, "!!!!! Erreur lors de l'allocation mmoire de la matrice B sur le priphrique !!!!!\n");
		return EXIT_FAILURE;
	}
	//d_C
	status = cublasAlloc (N2, sizeof (d_C[0]), (void **) & d_C);
	if (status != CUBLAS_STATUS_SUCCESS)
	{
		fprintf (stderr, "!!!!! Erreur lors de l'allocation mmoire de la matrice C sur le priphrique !!!!!\n");
		return EXIT_FAILURE;
	}
	
	// Remplissage des matrices du priphrique avec les donnes des matrices de l'hte
	// avec gestion des erreurs
	status = cublasSetVector(N2, sizeof(h_A[0]), h_A, 1, d_A, 1);
	if (status != CUBLAS_STATUS_SUCCESS)
	{
		fprintf (stderr, "!!!!! Erreur  l'accs du priphrique lors de l'criture de la matrice A !!!!!\n");
		return EXIT_FAILURE;
	}
	
	status = cublasSetVector(N2, sizeof(h_B[0]), h_B, 1, d_B, 1);
	if (status != CUBLAS_STATUS_SUCCESS)
	{
		fprintf (stderr, "!!!!! Erreur  l'accs du priphrique lors de l'criture de la matrice A !!!!!\n");
		return EXIT_FAILURE;
	}
	
	status = cublasSetVector(N2, sizeof(h_C[0]), h_C, 1, d_C, 1);
	if (status != CUBLAS_STATUS_SUCCESS)
	{
		fprintf (stderr, "!!!!! Erreur  l'accs du priphrique lors de l'criture de la matrice A !!!!!\n");
		return EXIT_FAILURE;
	}
	
	// Nettoyage de la mmoire des erreurs
	cublasGetError();
	
	// Multiplication des matrices avec CuBLAS
	cublasSgemm('n', 'n', N, N, N, alpha, d_A, N, d_B, N, beta, d_C, N);
	
	// Gestion des erreurs
	status = cublasGetError();
	if (status != CUBLAS_STATUS_SUCCESS)
	{
		fprintf (stderr, "!!!! Erreur lors de l'excution du kernel CuBLAS !!!!!\n");
		return EXIT_FAILURE;
	}
	
	// Allocation de la mmoire sur l'hte pour la rcupration du rsultat depuis le priphrique
	// avec gestion des erreurs
	h_C = (float*)malloc(N2 * sizeof(h_C[0]));
	if (h_C == 0) {
		fprintf (stderr, "!!!!! Erreur lors de l'allocation de la mmoire sur l'hte pour la matrice C !!!!!\n");
		return EXIT_FAILURE;
	}
	
	// Lecture du rsultat
	// avec gestion des erreurs
	status = cublasGetVector(N2, sizeof(h_C[0]), d_C, 1, h_C, 1);
	if (status != CUBLAS_STATUS_SUCCESS) {
		fprintf (stderr, "!!!!! Erreur d'accs au priphrique (lecture de la matrice C) !!!!!\n");
		return EXIT_FAILURE;
	}
	
	// Libration de la mmoire sur l'hte
	free(h_A);
	free(h_B);
	free(h_C);
	
	// Libration de la mmoire sur le priphrique
	// avec gestion des erreurs
	status = cublasFree(d_A);
	if (status != CUBLAS_STATUS_SUCCESS)
	{
		fprintf (stderr, "!!!! Erreur lors de la libration de l'espace priphrique pour la matrice A !!!!!\n");
		return EXIT_FAILURE;
	}
	status = cublasFree(d_B);
	if (status != CUBLAS_STATUS_SUCCESS)
	{
		fprintf (stderr, "!!!! Erreur lors de la libration de l'espace priphrique pour la matrice B !!!!!\n");
		return EXIT_FAILURE;
	}
	status = cublasFree(d_C);
	if (status != CUBLAS_STATUS_SUCCESS)
	{
		fprintf (stderr, "!!!! Erreur lors de la libration de l'espace priphrique pour la matrice C !!!!!\n");
		return EXIT_FAILURE;
	}
	
	// Fermeture propre de CuBLAS
	status = cublasShutdown();
	if (status != CUBLAS_STATUS_SUCCESS)
	{
		fprintf (stderr, "!!!! Erreur  la fermeture de CuBLAS !!!!!\n");
		return EXIT_FAILURE;
	}
	
	printf("\n Appuyez sur ENTER pour quitter.\n");
	getchar();
	
	return EXIT_SUCCESS;
}