#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <cutil.h>

#include "cutil_kernel.cu"

float RandFloat(float low, float high)
{
    float t = (float)rand() / (float)RAND_MAX;
    return (1.0f - t) * low + t * high;
}

const int VECTOR_N  = 256;
const int ELEMENT_N = 4096;
const int    DATA_N = VECTOR_N * ELEMENT_N;
const int   DATA_SZ = DATA_N   * sizeof(float);
const int RESULT_SZ = VECTOR_N * sizeof(float);

int main()
{
    float *h_A, *h_B, *h_C_GPU;
    float *d_A, *d_B, *d_C;
    unsigned int hTimer;
    int i, j;
    
	int VECTOR_N  = 512;
	int    DATA_N = VECTOR_N;
	int   DATA_SZ = DATA_N    * sizeof(float);
	int RESULT_SZ = VECTOR_N  * sizeof(float);
    CUT_DEVICE_INIT();
    CUT_SAFE_CALL( cutCreateTimer(& hTimer) );

    printf("Initializing data...\n");
        printf("...allocating CPU memory.\n");
        h_A     = (float *) malloc(DATA_SZ);
        h_B     = (float *) malloc(DATA_SZ);
        h_C_GPU = (float *) malloc(RESULT_SZ);

        printf("...allocating GPU memory.\n");
        CUDA_SAFE_CALL( cudaMalloc( (void **) & d_A, DATA_SZ)   );
        CUDA_SAFE_CALL( cudaMalloc( (void **) & d_B, DATA_SZ)   );
        CUDA_SAFE_CALL( cudaMalloc( (void **) & d_C, RESULT_SZ) );

        printf("...generating input data in CPU mem.\n");
        srand(153);
        for(i = 0 ; i < DATA_N ; i++)
		{
            h_A[i] = RandFloat(0.0f, 1.0f);
            h_B[i] = RandFloat(0.0f, 1.0f);
        }

        printf("...copying input data to GPU mem.\n");
        CUDA_SAFE_CALL( cudaMemcpy(d_A, h_A, DATA_SZ, cudaMemcpyHostToDevice) );
        CUDA_SAFE_CALL( cudaMemcpy(d_B, h_B, DATA_SZ, cudaMemcpyHostToDevice) );
    printf("Data init done.\n");

    printf("Executing GPU kernel...\n");
       CUDA_SAFE_CALL( cudaThreadSynchronize() );
        CUT_SAFE_CALL( cutResetTimer(hTimer) );
        CUT_SAFE_CALL( cutStartTimer(hTimer) );
        scalarProdGPU <<<1, VECTOR_N>>> (d_C, d_A, d_B, VECTOR_N);
        CUT_CHECK_ERROR("scalarProdGPU() execution failed\n");
        CUDA_SAFE_CALL ( cudaThreadSynchronize() );

    printf("Reading back GPU result...\n");
        CUDA_SAFE_CALL( cudaMemcpy(h_C_GPU, d_C, RESULT_SZ, cudaMemcpyDeviceToHost) );
        CUT_SAFE_CALL ( cutStopTimer(hTimer) );
    printf("GPU time: %f msecs.\n", cutGetTimerValue(hTimer));
    
    printf("Shutting down...\n");
	    CUDA_SAFE_CALL( cudaFree(d_C) );
	    CUDA_SAFE_CALL( cudaFree(d_B)   );
	    CUDA_SAFE_CALL( cudaFree(d_A)   );
	    free(h_C_GPU);
	    free(h_B);
	    free(h_A);
	    CUT_SAFE_CALL( cutDeleteTimer(hTimer) );
}